"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const Http = require("http");
class Dispatcher {
    constructor(port, handlers) {
        this.handlers = handlers;
        this.port = port;
    }
    run() {
        let server = this.getServer();
        server.listen(this.port, (error) => {
            if (error) {
                return console.log("Error", error);
            }
            console.log("Dispatcher running on port " + this.port);
        });
    }
    getServer() {
        return Http.createServer((request, response) => {
            console.log("Received request [ " + request.method + " ]@[ " + request.url + " ]");
            for (let handler of this.handlers) {
                if (handler.canHandleRequest(request)) {
                    console.log("Found suitable handler for request: [ " + handler.action.name + " ]");
                    handler.handleRequest(response);
                    return;
                }
            }
            console.log("No suitable handler found for request: [ " + request.method + " ]@[ " + request.url + " ]");
            response.end();
        });
    }
}
exports.Dispatcher = Dispatcher;
