"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RequestMethod_1 = require("../enum/RequestMethod");
class Handler {
    constructor(requestUrl, requestMethod, action, jsonGenerator) {
        this._requestUrl = requestUrl;
        this._requestMethod = requestMethod;
        this._action = action;
        this._jsonGenerator = jsonGenerator;
    }
    canHandleRequest(request) {
        let requestUrl = request.url;
        let requestMethod = RequestMethod_1.RequestMethod[request.method];
        return requestUrl === this._requestUrl &&
            (this._requestMethod === RequestMethod_1.RequestMethod.ANY ||
                requestMethod === this._requestMethod);
    }
    handleRequest(response) {
        let handlerResult = Reflect.apply(this._action, this, []);
        if (this._jsonGenerator) {
            response.writeHead(200, { "Content-Type": "application/json" });
            response.end(JSON.stringify(handlerResult));
        }
        response.end(handlerResult);
    }
    get requestUrl() {
        return this._requestUrl;
    }
    set requestUrl(value) {
        this._requestUrl = value;
    }
    get requestMethod() {
        return this._requestMethod;
    }
    set requestMethod(value) {
        this._requestMethod = value;
    }
    get action() {
        return this._action;
    }
    set action(value) {
        this._action = value;
    }
    get jsonGenerator() {
        return this._jsonGenerator;
    }
    set jsonGenerator(value) {
        this._jsonGenerator = value;
    }
}
exports.Handler = Handler;
