"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Key_1 = require("../constant/Key");
const RequestMethod_1 = require("../enum/RequestMethod");
class MetadataProcessor {
    static processControllers(controllers) {
        let handlers = [];
        console.log("Started controllers analysis");
        for (let controller of controllers) {
            let controllerClassName = controller.constructor.name;
            console.log("Started [ " + controllerClassName + " ] analysis");
            let controllerPrototype = Reflect.getPrototypeOf(controller);
            if (!Reflect.hasMetadata(Key_1.Key.METADATA_CONTROLLER_BASE_PATH, controllerPrototype)) {
                console.warn("Skipping [ " + controllerClassName + " ] analysis: not a valid controller instance");
                continue;
            }
            let controllerHandlers = Reflect.getMetadata(Key_1.Key.METADATA_CONTROLLER_HANDLERS, controllerPrototype);
            handlers.push(...controllerHandlers);
            for (let controllerHandler of controllerHandlers) {
                console.log("Found handler [ " + controllerHandler.action.name + " ] " +
                    "responding to [ " + RequestMethod_1.RequestMethod[controllerHandler.requestMethod] + " ]" +
                    "@[ " + controllerHandler.requestUrl + " ]");
            }
            console.log("Finished analyzing [ " + controllerClassName + " ]");
        }
        console.log("Finished analyzing controllers");
        return handlers;
    }
}
exports.MetadataProcessor = MetadataProcessor;
