"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const RequestMethod_1 = require("../enum/RequestMethod");
const Handler_1 = require("../core/Handler");
require("reflect-metadata");
const util_1 = require("util");
const Key_1 = require("../constant/Key");
function RequestHandler(requestUri, requestMethod) {
    return function (target, propertyKey, propertyDescriptor) {
        requestUri = util_1.isNullOrUndefined(requestUri) ? "/" : requestUri;
        requestMethod = util_1.isNullOrUndefined(requestMethod) ? RequestMethod_1.RequestMethod.ANY : requestMethod;
        let existingHandlers = [];
        if (Reflect.hasMetadata(Key_1.Key.METADATA_CONTROLLER_HANDLERS, target)) {
            existingHandlers = Reflect.getMetadata(Key_1.Key.METADATA_CONTROLLER_HANDLERS, target);
        }
        let jsonGeneratorKeys = [];
        if (Reflect.hasMetadata(Key_1.Key.METADATA_CONTROLLER_JSON_GENERATOR_KEYS, target)) {
            jsonGeneratorKeys = Reflect.getMetadata(Key_1.Key.METADATA_CONTROLLER_JSON_GENERATOR_KEYS, target);
        }
        let handler = new Handler_1.Handler(requestUri, requestMethod, propertyDescriptor.value, jsonGeneratorKeys.indexOf(propertyKey) > -1);
        existingHandlers.push(handler);
        Reflect.defineMetadata(Key_1.Key.METADATA_CONTROLLER_HANDLERS, existingHandlers, target);
    };
}
exports.RequestHandler = RequestHandler;
