import { Handler } from "./Handler";
import "reflect-metadata";
import * as Http from "http";
import { RequestMethod } from "../enum/RequestMethod";
import { Server } from "http";

export class Dispatcher {

    private port: number;
    private handlers: Handler[];

    constructor( port: number, handlers: Handler[] ) {
        this.handlers = handlers;
        this.port = port;
    }

    public run(): void {

        let server: Server = this.getServer();
        server.listen( this.port, ( error ) => {

            if( error ) {
                return console.log( "Error", error );
            }
            console.log( "Dispatcher running on port " + this.port );

        } );

    }

    private getServer(): Server {

        return Http.createServer( ( request, response ) => {

            console.log( "Received request [ " + request.method + " ]@[ " +  request.url + " ]" );
            for( let handler of this.handlers ) {

                if( handler.canHandleRequest( request ) ) {
                    console.log( "Found suitable handler for request: [ " + handler.action.name + " ]" );
                    handler.handleRequest( response );
                    return;
                }

            }
            console.log( "No suitable handler found for request: [ " + request.method + " ]@[ " +  request.url + " ]" );
            response.end();

        } );

    }

}