import { RequestMethod } from "../enum/RequestMethod";

export class Handler {

    private _requestUrl: string;
    private _requestMethod: RequestMethod;
    private _action: Function;
    private _jsonGenerator: boolean;

    constructor(
            requestUrl: string,
            requestMethod: RequestMethod,
            action: Function,
            jsonGenerator: boolean ) {

        this._requestUrl = requestUrl;
        this._requestMethod = requestMethod;
        this._action = action;
        this._jsonGenerator = jsonGenerator;

    }

    public canHandleRequest( request: any ) {

        let requestUrl: string = request.url;
        let requestMethod: RequestMethod = RequestMethod[ request.method as string ];
        return requestUrl === this._requestUrl &&
               ( this._requestMethod === RequestMethod.ANY ||
                 requestMethod === this._requestMethod );

    }

    public handleRequest( response: any ): void {

        let handlerResult: any = Reflect.apply( this._action, this, [] );
        if( this._jsonGenerator ) {
            response.writeHead( 200, { "Content-Type": "application/json" } );
            response.end( JSON.stringify( handlerResult ) );
        }
        response.end( handlerResult );

    }


    public get requestUrl(): string {
        return this._requestUrl;
    }

    public set requestUrl( value: string ) {
        this._requestUrl = value;
    }

    public get requestMethod(): RequestMethod {
        return this._requestMethod;
    }

    public set requestMethod( value: RequestMethod ) {
        this._requestMethod = value;
    }

    public get action(): Function {
        return this._action;
    }

    public set action( value: Function ) {
        this._action = value;
    }

    public get jsonGenerator(): boolean {
        return this._jsonGenerator;
    }

    public set jsonGenerator( value: boolean ) {
        this._jsonGenerator = value;
    }

}