import { Handler } from "./Handler";
import { Key } from "../constant/Key";
import { RequestMethod } from "../enum/RequestMethod";

export class MetadataProcessor {

    public static processControllers( controllers: Object[] ): Handler[] {

        let handlers: Handler[] = [];
        console.log( "Started controllers analysis" );
        for( let controller of controllers ) {

            let controllerClassName = controller.constructor.name;
            console.log( "Started [ " + controllerClassName + " ] analysis" );

            let controllerPrototype: Object = Reflect.getPrototypeOf( controller );
            if( !Reflect.hasMetadata( Key.METADATA_CONTROLLER_BASE_PATH, controllerPrototype ) ) {
                console.warn( "Skipping [ " + controllerClassName + " ] analysis: not a valid controller instance" );
                continue;
            }

            let controllerHandlers: Handler[] = Reflect.getMetadata(
                Key.METADATA_CONTROLLER_HANDLERS,
                controllerPrototype
            );
            handlers.push( ...controllerHandlers );

            for( let controllerHandler of controllerHandlers ) {
                console.log(
                    "Found handler [ " + controllerHandler.action.name + " ] " +
                    "responding to [ " + RequestMethod[ controllerHandler.requestMethod ] + " ]" +
                    "@[ " + controllerHandler.requestUrl + " ]"
                );
            }

            console.log( "Finished analyzing [ " + controllerClassName + " ]" );

        }
        console.log( "Finished analyzing controllers" );
        return handlers;

    }

}