import { isNullOrUndefined } from "util";
import "reflect-metadata";
import { Key } from "../constant/Key";
import { Handler } from "../core/Handler";

export function Controller( basePath?: string ) {

    basePath = isNullOrUndefined( basePath ) ? "/" : basePath;
    return function( target: Function ) {

        let handlers: Handler[] = [];

        if( Reflect.hasMetadata( Key.METADATA_CONTROLLER_HANDLERS, target.prototype ) ) {

            handlers = Reflect.getMetadata(
                Key.METADATA_CONTROLLER_HANDLERS,
                target.prototype
            );
            for( let handler of handlers ) {

                let handlerRequestUrl: string = handler.requestUrl;
                handlerRequestUrl = ( basePath + handlerRequestUrl );
                handlerRequestUrl.replace( "//", "/" );
                let handlerRequestUrlLastPosition: number = handlerRequestUrl.length - 1;
                if( handlerRequestUrl.charAt( handlerRequestUrlLastPosition ) === "/" ) {
                    handlerRequestUrl =
                        handlerRequestUrl.substring( 0, handlerRequestUrlLastPosition );
                }
                handler.requestUrl = handlerRequestUrl;

            }

        }
        Reflect.defineMetadata(
            Key.METADATA_CONTROLLER_HANDLERS,
            handlers,
            target.prototype
        );

        Reflect.defineMetadata(
            Key.METADATA_CONTROLLER_BASE_PATH,
            basePath,
            target.prototype
        );

    }

}