import { RequestMethod } from "../enum/RequestMethod";
import { Handler } from "../core/Handler";
import "reflect-metadata";
import { isNullOrUndefined } from "util";
import { Key } from "../constant/Key";

export function RequestHandler( requestUri?: string, requestMethod?: RequestMethod ) {

    return function( target: any, propertyKey: string, propertyDescriptor: PropertyDescriptor ) {

        requestUri = isNullOrUndefined( requestUri ) ? "/" : requestUri;
        requestMethod = isNullOrUndefined( requestMethod ) ? RequestMethod.ANY : requestMethod;

        let existingHandlers: Handler[] = [];
        if( Reflect.hasMetadata( Key.METADATA_CONTROLLER_HANDLERS, target ) ) {
            existingHandlers = Reflect.getMetadata( Key.METADATA_CONTROLLER_HANDLERS, target );
        }

        let jsonGeneratorKeys: string[] = [];
        if( Reflect.hasMetadata( Key.METADATA_CONTROLLER_JSON_GENERATOR_KEYS, target ) ) {
            jsonGeneratorKeys = Reflect.getMetadata(
                Key.METADATA_CONTROLLER_JSON_GENERATOR_KEYS,
                target
            );
        }

        let handler: Handler = new Handler(
            requestUri,
            requestMethod,
            propertyDescriptor.value,
            jsonGeneratorKeys.indexOf( propertyKey ) > -1
        );
        existingHandlers.push( handler );
        Reflect.defineMetadata(
            Key.METADATA_CONTROLLER_HANDLERS,
            existingHandlers,
            target
        );

    };

}